/*
 * MINIGUI - Harbour Win32 GUI library Demo
 * MiniGUI OLE Excel Demo	 
 *
 * Copyright 2015-2019 Verchenko Andrey <verchenkoag@gmail.com>
 * Copyright 2015 Grigory Filatov <gfilatov@inbox.ru>
 * 
 * .  DBF  XLS     .
 * Example. Convert DBF to XLS via the OLE object in MiniGui
*/

#include "minigui.ch"
#include "metrocolor.ch"

#define PROGRAM     "Convert DBF to XLS via the OLE object / " + MsgAboutDim(5)

REQUEST HB_CODEPAGE_RU1251, HB_CODEPAGE_RU866
REQUEST DBFCDX, DBFFPT

PROCEDURE MAIN
   LOCAL nWidth, nHeight, aBackColor, nLbl0, nLblCol
   LOCAL nJ, aDim[3], cFont := "Tahona", nFontSize := 14
   LOCAL cFileDbf := GetStartUpFolder() +"\test.dbf"
   LOCAL cFileXls := GetStartUpFolder() +"\test.xls"

   SET LANGUAGE TO RUSSIAN  
   SET CODEPAGE TO RUSSIAN  
   
   _HMG_MESSAGE[4] := "    :" + CRLF + ; 
                      App.ExeName + CRLF + ; 
                      "  ." + CRLF + _HMG_MESSAGE[4]    
   SET MULTIPLE OFF WARNING 

   DEFINE FONT DlgFont FONTNAME "Verdana" SIZE nFontSize 
   SET MSGALERT FONTCOLOR TO BLACK

   RDDSETDEFAULT('DBFCDX')

   SET FONT TO cFont , nFontSize 
   
   IF !hb_mtvm()
      MsgStop ("No support for multi-threading!" + CRLF + CRLF + ;
                "Compiling with a key /mt" + CRLF )
      QUIT
   ENDIF
   
   aDim[1] := "    "  
   aDim[2] := "  OLE Excel"  
   aDim[3] := MiniGUIVersion()  
 
   DEFINE WINDOW Forma_1 AT 0,0       ;
       WIDTH 700 HEIGHT 500           ;
       MAIN                           ;
       TITLE PROGRAM                  ; 
       ICON "1MAIN_ICO"               ;
       BACKCOLOR COLOR_LIGHT_GREEN    ;
       NOMAXIMIZE NOSIZE              ;
       ON INIT { || OnInitForma1() }  ;
       ON RELEASE { || OnCloseForma1(cFileDbf,cFileXls) } 

       nWidth  := Forma_1.Width
       nHeight := Forma_1.Height 
       aBackColor := COLOR_GREEN_METRO

       nLbl0 := 100
       @ 0, 0 LABEL Label_0 WIDTH nWidth HEIGHT nLbl0 VALUE "" ;
         FONT cFont SIZE nFontSize BOLD  CENTERALIGN ;
         FONTCOLOR aBackColor BACKCOLOR aBackColor  VCENTERALIGN

       @ 2, 20 IMAGE Image_1 PICTURE 'LOGO' WIDTH 96 HEIGHT 96 STRETCH ;
         TRANSPARENT BACKGROUNDCOLOR aBackColor ;
         ACTION MsgAbout() OnMouseHover RC_CURSOR( "MINIGUI_FINGER" )

       nJ := nLbl0/3 
       nLblCol := Forma_1.Image_1.Width + Forma_1.Image_1.Col + 5
       @ 0, nLblCol LABEL Label_1 WIDTH nWidth-nLblCol HEIGHT nJ VALUE aDim[1] ;
          FONT cFont SIZE nFontSize*1.3 BOLD  CENTERALIGN ;
          FONTCOLOR YELLOW BACKCOLOR aBackColor  VCENTERALIGN
       @ nJ*1, nLblCol LABEL Label_2 WIDTH nWidth-nLblCol HEIGHT nJ VALUE aDim[2] ;
          FONT cFont SIZE nFontSize*1.3 BOLD  CENTERALIGN ;
          FONTCOLOR YELLOW BACKCOLOR aBackColor  VCENTERALIGN
       @ nJ*2, nLblCol LABEL Label_3 WIDTH nWidth-nLblCol HEIGHT nJ VALUE aDim[3] ;
          FONT cFont SIZE nFontSize BOLD  CENTERALIGN ;
          FONTCOLOR YELLOW BACKCOLOR aBackColor  VCENTERALIGN

       @ nLbl0 + 20, 20  LABEL Label_Excel VALUE "Version Excel" ;
         WIDTH nWidth-40 HEIGHT 60 FONTCOLOR BLACK BOLD TRANSPARENT 

       @ 200,40 BUTTONEX Button_Dbf CAPTION '' + CRLF +'Dbf-';
          WIDTH 360 HEIGHT 110 BACKCOLOR COLOR_DARK_BLUE FONTCOLOR WHITE BOLD ;
          ICON "Dbf64" NOXPSTYLE HANDCURSOR ACTION {|| Forma_1.Minimize, DbfNew(cFileDbf) }

       @ 320,40 BUTTONEX Button_Xls CAPTION '' + CRLF +'Xls-' + CRLF +' Dbf-';
          WIDTH 360 HEIGHT 110 BACKCOLOR COLOR_BRIGHT_GREEN FONTCOLOR WHITE BOLD ;
          ICON "Excel64" NOXPSTYLE HANDCURSOR ACTION {|| Forma_1.Minimize, XlsReport(cFileDbf,cFileXls) }

       @ 200,410 BUTTONEX Button_Help CAPTION ' ' ;
          WIDTH 250 HEIGHT 110 BACKCOLOR COLOR_YELLOW_METRO FONTCOLOR WHITE BOLD ;
          ICON "help64" NOXPSTYLE HANDCURSOR ACTION  {|| MsgAbout() }

       @ 320,410 BUTTONEX Button_Exit CAPTION ' ' + CRLF +'' ;
          WIDTH 250 HEIGHT 110 BACKCOLOR COLOR_RED_METRO FONTCOLOR WHITE BOLD ;
          ICON "Close64" NOXPSTYLE HANDCURSOR ACTION  {|| ThisWindow.Release }

   END WINDOW

   CENTER WINDOW Forma_1
   ACTIVATE WINDOW Forma_1

RETURN

////////////////////////////////////////////////////////////
FUNCTION OnInitForma1()
  LOCAL cInfo, cPath, aRet := {1,""}

  aRet := MyExcelVersion()
  cInfo := ":  " + aRet[2] + "  (" + HB_NtoS(aRet[1]) + ")" + CRLF

  cPath := MyExcelPath()
  cPath := IIF( cPath == NIL, "???????", cPath)
  cInfo += "  Excel:  " + cPath

  Forma_1.Label_Excel.Value := cInfo
  Forma_1.Image_1.Setfocus

RETURN NIL

////////////////////////////////////////////////////////////
// Here you can delete unwanted files on program exit
FUNCTION OnCloseForma1(cFileDbf,cFileXls)  
   DEFAULT cFileDbf := "" , cFileXls := ""
   DELETEFILE(cFileDbf)
   DELETEFILE(cFileXls)
RETURN NIL

////////////////////////////////////////////////////////////
FUNCTION DbfNew(cFileDbf)
   LOCAL cMsg, nTime

   nTime := SECONDS()

   Forma_1.Button_Dbf.Enabled := .F. //   / lock button

   // Create the window waiting with the flow
   //     
   WaitThreadCreate( ' dbf- ...' )

   // Main loop calculations. 
   //   . 
   MyTestDbf(cFileDbf)

   WaitThreadClose()   //   "" / close the "expectations"

   cMsg := cFileDbf + ";;    - " + SECTOTIME( SECONDS() - nTime ) + " !"
   MyInfo(cMsg)

   Forma_1.Button_Dbf.Enabled := .T. //   / unlock button
   Forma_1.Restore  //      

   RETURN NIL

////////////////////////////////////////////////////////////
FUNCTION XlsReport(cFileDbf,cFileXls)
   LOCAL cMsg, nTime, cTitle, lActivate, lSave
   LOCAL nOrder, aField, aHatShapes, aFooting, cList
   LOCAL cFileIndex := cFileDbf + ".cdx"

   IF !FILE(cFileDbf)
      cMsg := " !    !;;" 
      cMsg += cFileDbf + ";;"
      cMsg += "   !;   !" 
      MyStop(cMsg)
      RETURN NIL
   ENDIF
   nTime := SECONDS()

   Forma_1.Button_Xls.Enabled := .F. //   / lock button
                                                   
   WaitThreadCreate( "  ..." )   //   -> WaitThread.prg

   USE ( cFileDbf ) Alias COMPANY CODEPAGE "RU1251" VIA "DBFCDX" NEW EXCLUSIVE
   INDEX ON "UPPER(UCOMP)" TAG MAIN TO ( cFileIndex ) 
   ORDLISTADD( cFileIndex )
   nOrder := 1

   cTitle    := "  ()   2   2014."
   lActivate := .T.
   lSave     := .T.
   aHatShapes:= { WHITE, COLOR_GREEN_METRO, 18 } //   
   aFooting  := {"","/ ../"}    //  
   cList     := " "   //  " 1"  

   aField := {}   //     
   AADD( aField, { "ORDKEYNO()", ""                             } ) 
   AADD( aField, { "UCOMP"     , " "                 } ) 
   AADD( aField, { "NACISL"    , "; ;(.)"        } ) 
   AADD( aField, { "PRIXOD"    , "; ;(.)"         } ) 
   AADD( aField, { "ITOGO"     , ";(.)"                    } ) 
   AADD( aField, { "SUMDOG"    , ";-;"         } ) 
   AADD( aField, { "KOLVO"     , ";-;"            } ) 

   WaitThreadClose()  //     -> WaitThread.prg

   MyExcelOle( "COMPANY", nOrder, aField, cFileXls, lActivate, cTitle, lSave,;
                aHatShapes, aFooting, cList )
  
   CLOSE COMPANY
   DELETEFILE(cFileIndex)

   cMsg := cFileXls + ";;    - " + SECTOTIME( SECONDS() - nTime ) + " !"
   MyInfo(cMsg)

   Forma_1.Button_Xls.Enabled := .T. //   / unlock button
   //Forma_1.Restore  //      

   RETURN NIL

////////////////////////////////////////////////////////////
FUNCTION MyTestDbf(cFileDbf)
   LOCAL aDbf := {}, nI, aDim := {}

   AADD( aDbf , {"KUCOMP" ,"N",  5, 0 } )
   AADD( aDbf , {"UCOMP " ,"C", 50, 0 } )
   AADD( aDbf , {"NACISL" ,"N", 14, 2 } )
   AADD( aDbf , {"PRIXOD" ,"N", 14, 2 } )
   AADD( aDbf , {"ITOGO " ,"N", 14, 2 } )
   AADD( aDbf , {"SUMDOG" ,"N",  6, 0 } )
   AADD( aDbf , {"KOLVO"  ,"N",  6, 0 } )
   AADD( aDbf , {"DATE2"  ,"D",  8, 0 } ) 
   AADD( aDbf , {"ID"     ,"+",  8, 0 } )

   AADD( aDim , {  1,'0701                    ',75589.39, 94770.6, 1,130,2021 } )
   AADD( aDim , {  2,'0701100000            ',68303.56, 19262.2, 4,314,7502 } )
   AADD( aDim , {  3,'070200000    ',       0,       0, 0,  0,   0 } ) 
   AADD( aDim , {  4,'0703  ,      ',10028.99,47531.89, 9,217,5553 } )
   AADD( aDim , {  5,'0703200000                       ', 1087.52, 1288.94, 2,  2,  43 } )
   AADD( aDim , {  7,'0703900000 -            ',56801.77,83944.86, 9,243,4613 } )
   AADD( aDim , {  6,'0704100000      ',72519.14, 90663.4, 6, 93,2240 } )
   AADD( aDim , {  8,'0704200000          ', 5870.00, 8204.66, 6,120,2797 } )
   AADD( aDim , { 10,'070700                 ',18068.84,21015.44, 6, 24, 413 } )
   AADD( aDim , { 13,'0708                       ',       0,       0, 0,  0,   0 } )
   AADD( aDim , { 14,'0708100000  (Pisum sativum)       ',       0,       0, 0,  4, 168 } )
   AADD( aDim , { 19,'0708200000                       ',    4560,    5040, 0,  1, 111 } )
   AADD( aDim , { 12,'0708900000           ',39065.03, 5227.73, 7,332,8974 } )
   AADD( aDim , { 15,'0709200000                       ',    4288,    4752, 6, 13, 105 } )
   AADD( aDim , { 16,'0709300000  ()      ',    3840, 4372.73, 3,  2,  84 } )
   AADD( aDim , { 17,'070610000                  ', 1412.84, 1750.58, 4, 12, 301 } )
   AADD( aDim , { 18,'0710400000             ',    7200, 8648.47, 7, 20, 169 } )
                                                                                         
   DBCreate( cFileDbf, aDbf , "DBFCDX")
   USE ( cFileDbf ) Alias TMP2 CODEPAGE "RU1251" VIA "DBFCDX" NEW EXCLUSIVE
   FOR nI := 1 TO LEN(aDim)
      APPEND BLANK
      TMP2->KUCOMP := aDim[nI,1] 
      TMP2->UCOMP  := aDim[nI,2]
      TMP2->NACISL := aDim[nI,3]
      TMP2->PRIXOD := aDim[nI,4]
      TMP2->ITOGO  := aDim[nI,4] - aDim[nI,3]
      TMP2->SUMDOG := aDim[nI,5]
      TMP2->KOLVO  := aDim[nI,6]
      TMP2->DATE2  := DATE() 
      INKEYGUI(200)
   NEXT
   CLOSE TMP2

   RETURN NIL

//////////////////////////////////////////////////////////////////////////
FUNCTION MyStop(cMsg,cTitle) 
   LOCAL bInit := {|| this.topmost := .t. }
   DEFAULT cTitle := ""

   SET MSGALERT FONTCOLOR TO MAROON
   SET MSGALERT BACKCOLOR TO { 255, 178, 178 }

   cMsg := AtRepl( ";", cMsg, CRLF )

   HMG_Alert( cMsg, {"&"}, cTitle, Nil, "Stop64", 64, { {189, 30, 73} } , bInit ) 

   RETURN NIL

//////////////////////////////////////////////////////////////////////////
FUNCTION MyInfo(cMsg,cTitle) 
   LOCAL bInit := {|| this.topmost := .t., ThisWindow.Restore }
   DEFAULT cTitle := ""

   SET MSGALERT FONTCOLOR TO BLACK
   SET MSGALERT BACKCOLOR TO { 141, 179, 226 }   

   cMsg := AtRepl( ";", cMsg, CRLF )

   HMG_Alert( cMsg, {"&"}, cTitle, Nil, "Smile64", 64, { {84,141,212} } , bInit ) 

   RETURN NIL

//////////////////////////////////////////////////////////////////////////
FUNCTION MyExclam(cMsg,cTitle) 
   LOCAL bInit := {|| this.topmost := .t. }
   DEFAULT cTitle := ""

   SET MSGALERT FONTCOLOR TO BLACK
   SET MSGALERT BACKCOLOR TO { 238, 249, 142 }   // -

   cMsg := AtRepl( ";", cMsg, CRLF )

   HMG_Alert( cMsg, {"&"}, cTitle, Nil, "Smile64", 64, { COLOR_YELLOW_METRO } , bInit ) 

   RETURN NIL
